@@ -397,17 +397,30 @@
 //! [`wasm-pack`]: https://github.com/rustwasm/wasm-pack/
 //! [`wasm-bindgen`]: https://github.com/rustwasm/wasm-bindgen
 
-#[cfg(all(not(feature = "sys"), not(feature = "js")))]
-compile_error!("At least the `sys` or the `js` feature must be enabled. Please, pick one.");
+#[cfg(all(not(feature = "sys"), not(feature = "js"), not(feature = "jsc")))]
+compile_error!("One of: `sys`, `js` or `jsc` features must be enabled. Please, pick one.");
 
 #[cfg(all(feature = "sys", feature = "js"))]
 compile_error!(
     "Cannot have both `sys` and `js` features enabled at the same time. Please, pick one."
 );
 
+#[cfg(all(feature = "js", feature = "jsc"))]
+compile_error!(
+    "Cannot have both `js` and `jsc` features enabled at the same time. Please, pick one."
+);
+
+#[cfg(all(feature = "sys", feature = "jsc"))]
+compile_error!(
+    "Cannot have both `sys` and `jsc` features enabled at the same time. Please, pick one."
+);
+
 #[cfg(all(feature = "sys", target_arch = "wasm32"))]
 compile_error!("The `sys` feature must be enabled only for non-`wasm32` target.");
 
+#[cfg(all(feature = "jsc", target_arch = "wasm32"))]
+compile_error!("The `jsc` feature must be enabled only for non-`wasm32` target.");
+
 #[cfg(all(feature = "js", not(target_arch = "wasm32")))]
 compile_error!(
     "The `js` feature must be enabled only for the `wasm32` target (either `wasm32-unknown-unknown` or `wasm32-wasi`)."
@@ -444,9 +457,15 @@ mod js;
 #[cfg(feature = "js")]
 pub use js::*;
 
+#[cfg(feature = "jsc")]
+mod jsc;
+
+#[cfg(feature = "jsc")]
+pub use jsc::*;
+
 pub use crate::externals::{Extern, Function, Global, HostFunction, Memory, MemoryView, Table};
 pub use access::WasmSliceAccess;
-pub use engine::{AsEngineRef, Engine};
+pub use engine::{AsEngineRef, Engine, EngineRef};
 pub use errors::{InstantiationError, LinkError, RuntimeError};
 pub use exports::{ExportError, Exportable, Exports, ExportsIterator};
 pub use extern_ref::ExternRef;
@@ -461,6 +480,8 @@ pub use ptr::{Memory32, Memory64, MemorySize, WasmPtr, WasmPtr64};
 pub use store::{AsStoreMut, AsStoreRef, OnCalledHandler, Store, StoreId, StoreMut, StoreRef};
 #[cfg(feature = "sys")]
 pub use store::{TrapHandlerFn, Tunables};
+#[cfg(any(feature = "sys", feature = "jsc"))]
+pub use target_lexicon::{Architecture, CallingConvention, OperatingSystem, Triple, HOST};
 pub use typed_function::TypedFunction;
 pub use value::Value;
 
@@ -470,10 +491,10 @@ pub use wasmer_derive::ValueType;
 // TODO: OnCalledAction is needed for asyncify. It will be refactored with https://github.com/wasmerio/wasmer/issues/3451
 pub use wasmer_types::{
     is_wasm, Bytes, CompileError, CpuFeature, DeserializeError, ExportIndex, ExportType,
-    ExternType, FunctionType, GlobalInit, GlobalType, ImportType, LocalFunctionIndex, MemoryError,
-    MemoryType, MiddlewareError, Mutability, OnCalledAction, Pages, ParseCpuFeatureError,
-    SerializeError, TableType, Target, Type, ValueType, WasmError, WasmResult, WASM_MAX_PAGES,
-    WASM_MIN_PAGES, WASM_PAGE_SIZE,
+    ExternType, FrameInfo, FunctionType, GlobalInit, GlobalType, ImportType, LocalFunctionIndex,
+    MemoryError, MemoryType, MiddlewareError, Mutability, OnCalledAction, Pages,
+    ParseCpuFeatureError, SerializeError, TableType, Target, Type, ValueType, WasmError,
+    WasmResult, WASM_MAX_PAGES, WASM_MIN_PAGES, WASM_PAGE_SIZE,
 };
 #[cfg(feature = "wat")]
 pub use wat::parse_bytes as wat2wasm;
