@@ -1,4 +1,5 @@
 pub(crate) mod engine;
+pub(crate) mod errors;
 pub(crate) mod extern_ref;
 pub(crate) mod externals;
 pub(crate) mod instance;
@@ -9,12 +10,11 @@ pub(crate) mod typed_function;
 
 pub use crate::sys::engine::NativeEngineExt;
 pub use crate::sys::tunables::BaseTunables;
-pub use target_lexicon::{Architecture, CallingConvention, OperatingSystem, Triple, HOST};
 #[cfg(feature = "compiler")]
 pub use wasmer_compiler::{
     wasmparser, CompilerConfig, FunctionMiddleware, MiddlewareReaderState, ModuleMiddleware,
 };
-pub use wasmer_compiler::{Artifact, EngineBuilder, Features, FrameInfo, Tunables};
+pub use wasmer_compiler::{Artifact, EngineBuilder, Features, Tunables};
 #[cfg(feature = "cranelift")]
 pub use wasmer_compiler_cranelift::{Cranelift, CraneliftOptLevel};
 #[cfg(feature = "llvm")]
@@ -34,4 +34,6 @@ pub(crate) mod vm {
     pub(crate) type VMExternMemory = InternalStoreHandle<VMMemory>;
     pub(crate) type VMExternGlobal = InternalStoreHandle<VMGlobal>;
     pub(crate) type VMExternFunction = InternalStoreHandle<VMFunction>;
+
+    pub type VMFunctionCallback = *const VMFunctionBody;
 }
