@@ -1,9 +1,12 @@
 use crate::DeserializeError;
-use std::str::FromStr;
 use std::string::ToString;
+use std::{
+    fmt::{self, Display, Formatter},
+    str::FromStr,
+};
 
 /// A hash used as a key when loading and storing modules in a
-/// [`Cache`].
+/// [`crate::Cache`].
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
 // Hash is made up of a 32 byte array
 pub struct Hash([u8; 32]);
@@ -11,7 +14,7 @@ pub struct Hash([u8; 32]);
 impl Hash {
     /// Creates a new instance from 32 raw bytes.
     /// Does not perform any hashing. In order to create a hash from data,
-    /// use `Hash::generate`.
+    /// use [`Hash::generate()`].
     pub fn new(bytes: [u8; 32]) -> Self {
         Self(bytes)
     }
@@ -21,17 +24,19 @@ impl Hash {
         let hash = blake3::hash(bytes);
         Self::new(hash.into())
     }
-
-    pub(crate) fn to_array(self) -> [u8; 32] {
-        self.0
-    }
 }
 
-impl ToString for Hash {
-    /// Create the hexadecimal representation of the
+impl Display for Hash {
+    /// Print the hexadecimal representation of the
     /// stored hash.
-    fn to_string(&self) -> String {
-        hex::encode(&self.to_array())
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        let mut buffer = [0_u8; 64];
+
+        hex::encode_to_slice(self.0, &mut buffer)
+            .expect("Can never fail with a hard-coded buffer length");
+        let s = std::str::from_utf8(&buffer).map_err(|_| fmt::Error)?;
+
+        f.write_str(s)
     }
 }
 
@@ -62,13 +67,16 @@ mod tests {
     use super::*;
 
     #[test]
-    fn hash_to_array_works() {
+    fn hash_is_displayed_as_hex() {
         let original = [
             0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF, 0x12, 0x65, 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF,
             0x12, 0x65, 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF, 0x12, 0x65, 0xAA, 0xBB, 0xCC, 0xDD,
             0xEE, 0xFF, 0x12, 0x65,
         ];
         let hash = Hash::new(original);
-        assert_eq!(hash.to_array(), original);
+        assert_eq!(
+            hash.to_string(),
+            "aabbccddeeff1265aabbccddeeff1265aabbccddeeff1265aabbccddeeff1265"
+        );
     }
 }
