@@ -1,4 +1,5 @@
 use clap::Parser;
+use wasmer_registry::WasmerConfig;
 
 #[derive(Debug, Parser)]
 /// The options for the `wasmer whoami` subcommand
@@ -11,7 +12,10 @@ pub struct Whoami {
 impl Whoami {
     /// Execute `wasmer whoami`
     pub fn execute(&self) -> Result<(), anyhow::Error> {
-        let (registry, username) = wasmer_registry::whoami(self.registry.as_deref())?;
+        let wasmer_dir =
+            WasmerConfig::get_wasmer_dir().map_err(|e| anyhow::anyhow!("no wasmer dir: {e}"))?;
+        let (registry, username) =
+            wasmer_registry::whoami(&wasmer_dir, self.registry.as_deref(), None)?;
         println!("logged into registry {registry:?} as user {username:?}");
         Ok(())
     }
