@@ -1,19 +1,19 @@
 //! This module mainly outputs the `Compiler` trait that custom
 //! compilers will need to implement.
 
-use crate::error::CompileError;
-use crate::function::Compilation;
 use crate::lib::std::boxed::Box;
 use crate::lib::std::sync::Arc;
-use crate::module::CompileModuleInfo;
-use crate::target::Target;
 use crate::translator::ModuleMiddleware;
 use crate::FunctionBodyData;
 use crate::ModuleTranslationState;
-use crate::SectionIndex;
-use loupe::MemoryUsage;
+use enumset::EnumSet;
+use wasmer_types::compilation::function::Compilation;
+use wasmer_types::compilation::module::CompileModuleInfo;
+use wasmer_types::compilation::symbols::SymbolRegistry;
+use wasmer_types::compilation::target::Target;
 use wasmer_types::entity::PrimaryMap;
-use wasmer_types::{Features, FunctionIndex, LocalFunctionIndex, SignatureIndex};
+use wasmer_types::error::CompileError;
+use wasmer_types::{CpuFeature, Features, LocalFunctionIndex};
 use wasmparser::{Validator, WasmFeatures};
 
 /// The compiler configuration options.
@@ -78,7 +78,12 @@ where
 }
 
 /// An implementation of a Compiler from parsed WebAssembly module to Compiled native code.
-pub trait Compiler: Send + MemoryUsage {
+pub trait Compiler: Send {
+    /// Returns a descriptive name for this compiler.
+    ///
+    /// Note that this is an API breaking change since 3.0
+    fn name(&self) -> &str;
+
     /// Validates a module.
     ///
     /// It returns the a succesful Result in case is valid, `CompileError` in case is not.
@@ -87,7 +92,6 @@ pub trait Compiler: Send + MemoryUsage {
         features: &Features,
         data: &'data [u8],
     ) -> Result<(), CompileError> {
-        let mut validator = Validator::new();
         let wasm_features = WasmFeatures {
             bulk_memory: features.bulk_memory,
             threads: features.threads,
@@ -95,13 +99,18 @@ pub trait Compiler: Send + MemoryUsage {
             multi_value: features.multi_value,
             simd: features.simd,
             tail_call: features.tail_call,
-            module_linking: features.module_linking,
             multi_memory: features.multi_memory,
             memory64: features.memory64,
             exceptions: features.exceptions,
             deterministic_only: false,
+            extended_const: features.extended_const,
+            relaxed_simd: features.relaxed_simd,
+            mutable_global: true,
+            saturating_float_to_int: true,
+            sign_extension: true,
+            component_model: false,
         };
-        validator.wasm_features(wasm_features);
+        let mut validator = Validator::new_with_features(wasm_features);
         validator
             .validate_all(data)
             .map_err(|e| CompileError::Validate(format!("{}", e)))?;
@@ -139,31 +148,9 @@ pub trait Compiler: Send + MemoryUsage {
 
     /// Get the middlewares for this compiler
     fn get_middlewares(&self) -> &[Arc<dyn ModuleMiddleware>];
-}
 
-/// The kinds of wasmer_types objects that might be found in a native object file.
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub enum Symbol {
-    /// A function defined in the wasm.
-    LocalFunction(LocalFunctionIndex),
-
-    /// A wasm section.
-    Section(SectionIndex),
-
-    /// The function call trampoline for a given signature.
-    FunctionCallTrampoline(SignatureIndex),
-
-    /// The dynamic function trampoline for a given function.
-    DynamicFunctionTrampoline(FunctionIndex),
-}
-
-/// This trait facilitates symbol name lookups in a native object file.
-pub trait SymbolRegistry: Send + Sync {
-    /// Given a `Symbol` it returns the name for that symbol in the object file
-    fn symbol_to_name(&self, symbol: Symbol) -> String;
-
-    /// Given a name it returns the `Symbol` for that name in the object file
-    ///
-    /// This function is the inverse of [`SymbolRegistry::symbol_to_name`]
-    fn name_to_symbol(&self, name: &str) -> Option<Symbol>;
+    /// Get the CpuFeatues used by the compiler
+    fn get_cpu_features_used(&self, cpu_features: &EnumSet<CpuFeature>) -> EnumSet<CpuFeature> {
+        *cpu_features
+    }
 }
