@@ -11,12 +11,11 @@
 //! is handled, according to the semantics of WebAssembly, to only specific expressions that are
 //! interpreted on the fly.
 use super::environ::ModuleEnvironment;
+use super::error::from_binaryreadererror_wasmerror;
 use super::state::ModuleTranslationState;
 use crate::wasm_unsupported;
-use crate::{WasmError, WasmResult};
 use core::convert::TryFrom;
 use std::boxed::Box;
-use std::collections::HashMap;
 use std::vec::Vec;
 use wasmer_types::entity::packed_option::ReservedValue;
 use wasmer_types::entity::EntityRef;
@@ -24,28 +23,25 @@ use wasmer_types::{
     DataIndex, ElemIndex, FunctionIndex, FunctionType, GlobalIndex, GlobalInit, GlobalType,
     MemoryIndex, MemoryType, Pages, SignatureIndex, TableIndex, TableType, Type, V128,
 };
+use wasmer_types::{WasmError, WasmResult};
 use wasmparser::{
     self, Data, DataKind, DataSectionReader, Element, ElementItem, ElementItems, ElementKind,
-    ElementSectionReader, Export, ExportSectionReader, ExternalKind, FuncType as WPFunctionType,
-    FunctionSectionReader, GlobalSectionReader, GlobalType as WPGlobalType, ImportSectionEntryType,
-    ImportSectionReader, MemorySectionReader, MemoryType as WPMemoryType, NameSectionReader,
-    Naming, NamingReader, Operator, TableSectionReader, TypeDef, TypeSectionReader,
+    ElementSectionReader, Export, ExportSectionReader, ExternalKind, FunctionSectionReader,
+    GlobalSectionReader, GlobalType as WPGlobalType, ImportSectionReader, MemorySectionReader,
+    MemoryType as WPMemoryType, NameSectionReader, Operator, TableSectionReader, TypeRef,
+    TypeSectionReader,
 };
 
 /// Helper function translating wasmparser types to Wasm Type.
-pub fn wptype_to_type(ty: wasmparser::Type) -> WasmResult<Type> {
+pub fn wptype_to_type(ty: wasmparser::ValType) -> WasmResult<Type> {
     match ty {
-        wasmparser::Type::I32 => Ok(Type::I32),
-        wasmparser::Type::I64 => Ok(Type::I64),
-        wasmparser::Type::F32 => Ok(Type::F32),
-        wasmparser::Type::F64 => Ok(Type::F64),
-        wasmparser::Type::V128 => Ok(Type::V128),
-        wasmparser::Type::ExternRef => Ok(Type::ExternRef),
-        wasmparser::Type::FuncRef => Ok(Type::FuncRef),
-        ty => Err(wasm_unsupported!(
-            "wptype_to_type: wasmparser type {:?}",
-            ty
-        )),
+        wasmparser::ValType::I32 => Ok(Type::I32),
+        wasmparser::ValType::I64 => Ok(Type::I64),
+        wasmparser::ValType::F32 => Ok(Type::F32),
+        wasmparser::ValType::F64 => Ok(Type::F64),
+        wasmparser::ValType::V128 => Ok(Type::V128),
+        wasmparser::ValType::ExternRef => Ok(Type::ExternRef),
+        wasmparser::ValType::FuncRef => Ok(Type::FuncRef),
     }
 }
 
@@ -59,15 +55,17 @@ pub fn parse_type_section(
     environ.reserve_signatures(count)?;
 
     for entry in types {
-        if let Ok(TypeDef::Func(WPFunctionType { params, returns })) = entry {
-            let sig_params: Vec<Type> = params
+        if let Ok(wasmparser::Type::Func(functype)) = entry {
+            let params = functype.params();
+            let returns = functype.results();
+            let sig_params: Box<[Type]> = params
                 .iter()
                 .map(|ty| {
                     wptype_to_type(*ty)
                         .expect("only numeric types are supported in function signatures")
                 })
                 .collect();
-            let sig_returns: Vec<Type> = returns
+            let sig_returns: Box<[Type]> = returns
                 .iter()
                 .map(|ty| {
                     wptype_to_type(*ty)
@@ -76,7 +74,9 @@ pub fn parse_type_section(
                 .collect();
             let sig = FunctionType::new(sig_params, sig_returns);
             environ.declare_signature(sig)?;
-            module_translation_state.wasm_types.push((params, returns));
+            module_translation_state
+                .wasm_types
+                .push((params.to_vec().into(), returns.to_vec().into()));
         } else {
             unimplemented!("module linking not implemented yet")
         }
@@ -93,25 +93,22 @@ pub fn parse_import_section<'data>(
     environ.reserve_imports(imports.get_count())?;
 
     for entry in imports {
-        let import = entry?;
+        let import = entry.map_err(from_binaryreadererror_wasmerror)?;
         let module_name = import.module;
-        let field_name = import.field;
+        let field_name = import.name;
 
         match import.ty {
-            ImportSectionEntryType::Function(sig) => {
+            TypeRef::Func(sig) => {
                 environ.declare_func_import(
                     SignatureIndex::from_u32(sig),
                     module_name,
-                    field_name.unwrap_or_default(),
+                    field_name,
                 )?;
             }
-            ImportSectionEntryType::Module(_) | ImportSectionEntryType::Instance(_) => {
-                unimplemented!("module linking not implemented yet")
-            }
-            ImportSectionEntryType::Tag(_) => {
+            TypeRef::Tag(_) => {
                 unimplemented!("exception handling not implemented yet")
             }
-            ImportSectionEntryType::Memory(WPMemoryType {
+            TypeRef::Memory(WPMemoryType {
                 shared,
                 memory64,
                 initial,
@@ -127,20 +124,20 @@ pub fn parse_import_section<'data>(
                         shared,
                     },
                     module_name,
-                    field_name.unwrap_or_default(),
+                    field_name,
                 )?;
             }
-            ImportSectionEntryType::Global(ref ty) => {
+            TypeRef::Global(ref ty) => {
                 environ.declare_global_import(
                     GlobalType {
                         ty: wptype_to_type(ty.content_type).unwrap(),
                         mutability: ty.mutable.into(),
                     },
                     module_name,
-                    field_name.unwrap_or_default(),
+                    field_name,
                 )?;
             }
-            ImportSectionEntryType::Table(ref tab) => {
+            TypeRef::Table(ref tab) => {
                 environ.declare_table_import(
                     TableType {
                         ty: wptype_to_type(tab.element_type).unwrap(),
@@ -148,7 +145,7 @@ pub fn parse_import_section<'data>(
                         maximum: tab.maximum,
                     },
                     module_name,
-                    field_name.unwrap_or_default(),
+                    field_name,
                 )?;
             }
         }
@@ -172,7 +169,7 @@ pub fn parse_function_section(
     environ.reserve_func_types(num_functions)?;
 
     for entry in functions {
-        let sigindex = entry?;
+        let sigindex = entry.map_err(from_binaryreadererror_wasmerror)?;
         environ.declare_func_type(SignatureIndex::from_u32(sigindex))?;
     }
 
@@ -187,7 +184,7 @@ pub fn parse_table_section(
     environ.reserve_tables(tables.get_count())?;
 
     for entry in tables {
-        let table = entry?;
+        let table = entry.map_err(from_binaryreadererror_wasmerror)?;
         environ.declare_table(TableType {
             ty: wptype_to_type(table.element_type).unwrap(),
             minimum: table.initial,
@@ -211,7 +208,7 @@ pub fn parse_memory_section(
             memory64,
             initial,
             maximum,
-        } = entry?;
+        } = entry.map_err(from_binaryreadererror_wasmerror)?;
         if memory64 {
             unimplemented!("64bit memory not implemented yet");
         }
@@ -239,9 +236,12 @@ pub fn parse_global_section(
                 mutable,
             },
             init_expr,
-        } = entry?;
+        } = entry.map_err(from_binaryreadererror_wasmerror)?;
         let mut init_expr_reader = init_expr.get_binary_reader();
-        let initializer = match init_expr_reader.read_operator()? {
+        let initializer = match init_expr_reader
+            .read_operator()
+            .map_err(from_binaryreadererror_wasmerror)?
+        {
             Operator::I32Const { value } => GlobalInit::I32Const(value),
             Operator::I64Const { value } => GlobalInit::I64Const(value),
             Operator::F32Const { value } => GlobalInit::F32Const(f32::from_bits(value.bits())),
@@ -280,19 +280,17 @@ pub fn parse_export_section<'data>(
 
     for entry in exports {
         let Export {
-            field,
+            name: field,
             ref kind,
             index,
-        } = entry?;
+        } = entry.map_err(from_binaryreadererror_wasmerror)?;
 
         // The input has already been validated, so we should be able to
         // assume valid UTF-8 and use `from_utf8_unchecked` if performance
         // becomes a concern here.
         let index = index as usize;
         match *kind {
-            ExternalKind::Function => {
-                environ.declare_func_export(FunctionIndex::new(index), field)?
-            }
+            ExternalKind::Func => environ.declare_func_export(FunctionIndex::new(index), field)?,
             ExternalKind::Table => environ.declare_table_export(TableIndex::new(index), field)?,
             ExternalKind::Memory => {
                 environ.declare_memory_export(MemoryIndex::new(index), field)?
@@ -300,9 +298,6 @@ pub fn parse_export_section<'data>(
             ExternalKind::Global => {
                 environ.declare_global_export(GlobalIndex::new(index), field)?
             }
-            ExternalKind::Type | ExternalKind::Module | ExternalKind::Instance => {
-                unimplemented!("module linking not implemented yet")
-            }
             ExternalKind::Tag => {
                 unimplemented!("exception handling not implemented yet")
             }
@@ -320,11 +315,17 @@ pub fn parse_start_section(index: u32, environ: &mut ModuleEnvironment) -> WasmR
 }
 
 fn read_elems(items: &ElementItems) -> WasmResult<Box<[FunctionIndex]>> {
-    let items_reader = items.get_items_reader()?;
+    let items_reader = items
+        .get_items_reader()
+        .map_err(from_binaryreadererror_wasmerror)?;
     let mut elems = Vec::with_capacity(usize::try_from(items_reader.get_count()).unwrap());
     for item in items_reader {
-        let elem = match item? {
-            ElementItem::Expr(init) => match init.get_binary_reader().read_operator()? {
+        let elem = match item.map_err(from_binaryreadererror_wasmerror)? {
+            ElementItem::Expr(init) => match init
+                .get_binary_reader()
+                .read_operator()
+                .map_err(from_binaryreadererror_wasmerror)?
+            {
                 Operator::RefNull { .. } => FunctionIndex::reserved_value(),
                 Operator::RefFunc { function_index } => FunctionIndex::from_u32(function_index),
                 s => {
@@ -354,8 +355,8 @@ pub fn parse_element_section<'data>(
             items,
             ty,
             range: _,
-        } = entry?;
-        if ty != wasmparser::Type::FuncRef {
+        } = entry.map_err(from_binaryreadererror_wasmerror)?;
+        if ty != wasmparser::ValType::FuncRef {
             return Err(wasm_unsupported!(
                 "unsupported table element type: {:?}",
                 ty
@@ -365,10 +366,13 @@ pub fn parse_element_section<'data>(
         match kind {
             ElementKind::Active {
                 table_index,
-                init_expr,
+                offset_expr,
             } => {
-                let mut init_expr_reader = init_expr.get_binary_reader();
-                let (base, offset) = match init_expr_reader.read_operator()? {
+                let mut init_expr_reader = offset_expr.get_binary_reader();
+                let (base, offset) = match init_expr_reader
+                    .read_operator()
+                    .map_err(from_binaryreadererror_wasmerror)?
+                {
                     Operator::I32Const { value } => (None, value as u32 as usize),
                     Operator::GlobalGet { global_index } => {
                         (Some(GlobalIndex::from_u32(global_index)), 0)
@@ -409,14 +413,17 @@ pub fn parse_data_section<'data>(
             kind,
             data,
             range: _,
-        } = entry?;
+        } = entry.map_err(from_binaryreadererror_wasmerror)?;
         match kind {
             DataKind::Active {
                 memory_index,
-                init_expr,
+                offset_expr,
             } => {
-                let mut init_expr_reader = init_expr.get_binary_reader();
-                let (base, offset) = match init_expr_reader.read_operator()? {
+                let mut init_expr_reader = offset_expr.get_binary_reader();
+                let (base, offset) = match init_expr_reader
+                    .read_operator()
+                    .map_err(from_binaryreadererror_wasmerror)?
+                {
                     Operator::I32Const { value } => (None, value as u32 as usize),
                     Operator::GlobalGet { global_index } => {
                         (Some(GlobalIndex::from_u32(global_index)), 0)
@@ -453,20 +460,20 @@ pub fn parse_name_section<'data>(
     while let Ok(subsection) = names.read() {
         match subsection {
             wasmparser::Name::Function(function_subsection) => {
-                if let Some(function_names) = function_subsection
-                    .get_map()
-                    .ok()
-                    .and_then(parse_function_name_subsection)
-                {
-                    for (index, name) in function_names {
-                        environ.declare_function_name(index, name)?;
+                for naming in function_subsection.into_iter().flatten() {
+                    if naming.index != std::u32::MAX {
+                        environ.declare_function_name(
+                            FunctionIndex::from_u32(naming.index),
+                            naming.name,
+                        )?;
                     }
                 }
             }
-            wasmparser::Name::Module(module) => {
-                if let Ok(name) = module.get_name() {
-                    environ.declare_module_name(name)?;
-                }
+            wasmparser::Name::Module {
+                name,
+                name_range: _,
+            } => {
+                environ.declare_module_name(name)?;
             }
             wasmparser::Name::Local(_) => {}
             wasmparser::Name::Label(_)
@@ -481,27 +488,3 @@ pub fn parse_name_section<'data>(
     }
     Ok(())
 }
-
-fn parse_function_name_subsection(
-    mut naming_reader: NamingReader<'_>,
-) -> Option<HashMap<FunctionIndex, &str>> {
-    let mut function_names = HashMap::new();
-    for _ in 0..naming_reader.get_count() {
-        let Naming { index, name } = naming_reader.read().ok()?;
-        if index == std::u32::MAX {
-            // We reserve `u32::MAX` for our own use.
-            return None;
-        }
-
-        if function_names
-            .insert(FunctionIndex::from_u32(index), name)
-            .is_some()
-        {
-            // If the function index has been previously seen, then we
-            // break out of the loop and early return `None`, because these
-            // should be unique.
-            return None;
-        }
-    }
-    Some(function_names)
-}
